@@ -705,10 +705,13 @@ read_portage_profile(const char *profile, env_vars vars[], set *masks)
 					/* empty repo name means a repo where the profile is */
 					const char* current_overlay = overlay_from_path (profile);
 					if (current_overlay == NULL) {
-						/* bring back the colon to see the ignored parent line */
+						/* bring back the colon to see the ignored
+						 * parent line */
 						*(--p) = ':';
-						warn("could not figure out current repo of profile %s, ignoring parent %s",
-							       profile, s);
+						if (!quiet)
+							warn("could not figure out current repo "
+								 "of profile %s, ignoring parent %s",
+							     profile, s);
 						continue;
 					}
 					snprintf(profile_file, sizeof(profile_file),
@@ -726,10 +729,12 @@ read_portage_profile(const char *profile, env_vars vars[], set *masks)
 						repo_name = NULL;
 					}
 					if (repo_name == NULL) {
-						/* bring back the colon to see the ignored parent line */
+						/* bring back the colon to see the ignored
+						 * parent line */
 						*(--p) = ':';
-						warn("ignoring parent with unknown repo in profile %s: %s",
-								profile, s);
+						if (!quiet)
+							warn("ignoring parent with unknown repo "
+								 "in profile %s: %s", profile, s);
 						continue;
 					}
                                 }
@@ -1052,6 +1057,12 @@ initialize_portage_env(void)
 			set_portage_env_var(var, s, var->name);
 	}
 
+	/* special snowflake, NO_COLOR is apparently some standard now,
+	 * accept it (as override of NOCOLOR) */
+	s = getenv("NO_COLOR");
+	if (s != NULL)
+		set_portage_env_var(&vars_to_read[7], s, "NO_COLOR");
+
 	/* expand any nested variables e.g. PORTDIR=${EPREFIX}/usr/portage */
 	for (i = 0; vars_to_read[i].name; ++i) {
 		char *svar;
