@@ -1,8 +1,8 @@
-#!/bin/bash
+#!/bin/env bash
 
 set -e
 
-if ! . /etc/init.d/functions.sh 2>/dev/null ; then
+if ! .  ${EPREFIX}/lib/gentoo/functions.sh 2>/dev/null ; then
 	einfo() { printf ' * %b\n' "$*"; }
 	eerror() { einfo "$@" 1>&2; }
 fi
@@ -26,46 +26,47 @@ git) ver="HEAD" ;;
 	fi
 	;;
 esac
-p="portage-utils-${ver#v}"
+p="${TMPDIR:-/var/tmp}/portage-utils-${ver#v}"
+pb=${p}_build
 
 rm -rf "${p}"
 mkdir "${p}"
+rm -rf "${pb}"
+mkdir "${pb}"
 
 einfo "Checking out clean git sources ..."
 git archive "${ver}" | tar xf - -C "${p}"
-cd "${p}"
+pushd "${p}" >/dev/null
 
 einfo "Building autotools ..."
 sed -i "/^AC_INIT/s:git:${ver#v}:" configure.ac
-sed -i "1iPV := ${ver#v}" Makefile
-LC_ALL=C ${MAKE} -s autotools >/dev/null
+sed -i "/^AM_MAINTAINER_MODE/s:(.*)$::" configure.ac
+./autogen.sh
 rm -rf autom4te.cache
-cd ..
-
-einfo "Generating tarball ..."
-tar --numeric-owner -cf - "${p}" | xz > "${p}".tar.xz
-rm -r "${p}"
-du -b "${p}".tar.*
-
-einfo "Checking tarball (simple) ..."
-tar xf "${p}".tar.*
-pushd "${p}" >/dev/null
-v ${MAKE} -s
-v ${MAKE} -s check
 popd >/dev/null
-rm -rf "${p}"
 
-einfo "Checking tarball (autotools) ..."
-tar xf "${p}".tar.*
-pushd "${p}" >/dev/null
+einfo "Checking sources ..."
+pushd "${pb}" >/dev/null
+tar cf - -C "${p}" . | tar xf -
 v ./configure -q
 v ${MAKE} -s
 v ${MAKE} -s check
+v ${MAKE} -s DESTDIR="${PWD}"/install install
+v ./man/mkman.py
+# ensure the tar contains up-to-date manpages
+cp -a man/*.1 "${p}"/man
 popd >/dev/null
-rm -rf "${p}"
+rm -rf "${pb}"
+
+einfo "Generating tarball ..."
+pushd "${p%/*}" >/dev/null
+tar --numeric-owner -cf - "${p##*/}" | xz > "${p}".tar.xz
+popd >/dev/null
+rm -r "${p}"
 
 echo
 einfo "All ready for distribution!"
-du -b "${p}".tar.*
+mv "${p}".tar.* .
+du -b "${p##*/}".tar.*
 
 exit 0
